/* 
 *  Copyright (C) 2006 Dip. Ing. dell'Informazione, University of Pisa, Italy
 *  http://info.iet.unipi.it/~cng/ns2measure/ns2measure.html
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA, USA
 */

/**
   project: measure
   filename: test_metrics.cc
	author: C. Cicconetti <c.cicconetti@iet.unipi.it>
	year: 2006
   affiliation:
      Dipartimento di Ingegneria dell'Informazione
	   University of Pisa, Italy
   description:
		test program for the classes in the measure.* files
*/

#include <config.h>
#include <stat.h>
#include <measure.h>

#include <cstdlib>
#include <iostream>
#include <vector>

using namespace std;

int
main (int argc, char* argv[]) {
	try {
		Metrics m;

		for ( unsigned int i = 0 ; i < 5 ; i++ ) {
			for ( double x = 1.0 ; x < 5.1 ; x += 1.0 ) {
				m.addSample ("tpt", x, i+2);
			}
		}
		for ( double x = 1.0 ; x < 5.1 ; x += 1.0 ) {
			m.addSample ("delay", x, 10);
		}
		m.addSample ("tpt", 1.0, 12);
		for ( unsigned int i = 0 ; i < 3 ; i++ ) {
			for ( double x = 1.0 ; x < 5.1 ; x += 1.0 ) {
				m.addSample ("tpt", x, i+2);
			}
		}

		for ( unsigned int i = 0 ; i < 5 ; i+=2 ) {
			for ( double x = 1.0 ; x < 5.1 ; x += 1.0 ) {
				for ( unsigned j = 0 ; j < 11 ; j++ ) {
					m.addSample ("delay", j/55.0, i+1, j);
				}
			}
		}
		m.addSample ("pollint", 0.0, 12, 0);
		m.addSample ("pollint", 1.0, 12, 1);
		m.addSample ("pollint", 0.5, 12, 0);
		m.addSample ("pollint", 0.5, 12, 1);

		m.setBinSize ("delay", .001);
		m.setDistLower ("delay", 0.0);
		m.setBinSize ("pollint", 1.0);
		m.setDistLower ("pollint", 10.0);

		m.dump (cout);
	} catch (Object& obj) {
		cout << "exception " << obj.getName() << "(" << obj.getId() << ")\n";
	}

	exit (0);
}
